#
#  Copyright 1996-2008 Jim Pirzyk / pirzyk.org
#  All rights reserved.
#
#  Security::OS::freebsd - FreeBSD specific configuration information
#
package Security::OS::freebsd;

use Exporter;
use strict;
use warnings;

our @ISA = qw(Exporter);
our @EXPORT_OK = qw( &init() &check() );

our (%perms, $mtree);

sub init {
	print ( STDERR  "In Security::OS::freebsd::init()\n") if ( $main::opts{'d'} );

	$main::showmount = '/usr/bin/showmount';
	$main::exportsfile = '/etc/exports';
	$main::securefile='/etc/ttys';
	$mtree = '/usr/sbin/mtree';
	$main::sudofile = '/usr/local/etc/sudoers';
	# Find the directory where the loadable kernel modules are
	my $sysctl = '/usr/sbin/sysctl' if ( -f '/usr/sbin/sysctl' );
	$sysctl = '/sbin/sysctl' if ( -f '/sbin/sysctl' );
	if ( -f $sysctl ) {
		chop ($main::kerneldir=`$sysctl -n kern.module_path 2> /dev/null`);
		$main::kerneldir =~ s,^/;,^,o;
		$main::kerneldir =~ s/;/|^/go;
	}
}

sub check {
	print ( STDERR  "In Security::OS::freebsd::check()\n") if ( $main::opts{'d'} );

	&Security::rootlogin::check_ttys() 
		if ( defined &Security::rootlogin::check_ttys );

	&check_osfiles();
}

sub check_osfiles {
	my ($dir, $dir2, $file);
	local *FP;

	print ( STDERR "In Security::OS::freebsd::check_osfiles()\n") if ( $main::opts{'d'} );

	if ( opendir (DIR, '/etc/mtree')) {  
		while ($file = readdir (DIR)) {
			$dir = '';
			$dir = '/'  if ( $file =~ /BSD.root.dist/ );
			$dir = '/var'  if ( $file =~ /BSD.var.dist/ );
			$dir = '/usr'  if ( $file =~ /BSD.usr.dist/ );
			$dir = '/usr/include'  if ( $file =~ /BSD.include.dist/ );
			$dir = '/usr/X11R6'  if ( $file =~ /BSD.x11.dist/ );
			$dir = '/usr/local'  if ( $file =~ /BSD.local.dist/ );
			if ( -d $dir && -f "/etc/mtree/$file" ) {
				print ( STDERR "\tRunning cmd $mtree -f /etc/mtree/$file -x -p $dir -e\n") if ( $main::opts{'d'} );
				open (FP, "$mtree -f /etc/mtree/$file -x -p $dir -e|") || &main::security_die ("Could not run $mtree\n");
				while (<FP>) {
					chop;
					if ( length $_ > 1 ) {
						if ( /missing$/o ) {
							$dir2 = $_;
							$dir2 =~ s, missing$,,o;
							$dir2 =~ s,^\.,,o;
							if ( $dir ne '/' ) {
								$dir2 = (( $dir2 =~ m|^/|)
									? "$dir$dir2"
									: "$dir/$dir2");
							}
							$perms{$dir2} = "file not found";
						} elsif ( /.* changed$/o ) {
							$dir2 = $_;
							$dir2 =~ s/ changed$//go;
							$dir2 =~ s,^\.,,o;
							if ( $dir ne '/' ) {
								$dir2 = (( $dir2 =~ m|^/|)
									? "$dir$dir2"
									: "$dir/$dir2");
							}
							$perms{$dir2} = 1;
						} elsif ( exists $perms{$dir2} ) {
							$perms{$dir2}=($perms{$dir2} eq '1')
								? $_: $perms{$dir2} . $_;
							$perms{$dir2} =~ s/  / /go;
						}
					}
				}
				close FP;
			}
		}
		closedir DIR;
	}
}

sub print {
	print ( STDERR  "In Security::OS::freebsd::print()\n") if ( $main::opts{'d'} );

	&main::print_data ("\n\nchecking mtree command output:\n", %perms);
}

1

__END__

=head1 NAME

Security::OS::freebsd

=head1 SYNOPSIS

Security::OS::freebsd::init();

Security::OS::freebsd::check();

Security::OS::freebsd::print();

=head1 DESCRIPTION

Sets variables for OS specific commands.

=head1 API

Defines the following API calls:

=over 4

=item C<init()>

Init sets the following global variables:

=over 4

=item C<$main::showmount>

The location of the showmount command, used in the C<Security::exports> module.

=item C<$main::exportsfile>

The location of the exports file, used in C<Security::exports> module.

=item C<$main::sudofile>

The location of F<sudoers> file, used in C<Security::sudo> module.

=item C<$main::kerneldir>

The location of the kernel loadable modules, used in C<Security::kernel>
module.

=item C<$main::securefile>

The location of the list of ports root is allowed in on, used in
C<Security::rootlogin> module.

=back

=item C<check()>

Calls C<Security::rootlogin::check_ttys()> to check what root ports are
configured and compares it to what ports we know about.

Calls L<mtree(8)> command to verify what file metadata matches what the
OS thinks they should be.

=item C<print()>

Prints out the L<mtree(8)> differences that we came across.

=back

=cut

